/*
    This file is part of the 'yarec' package.
    Copyright (C) 1998  Ralf W. Stephan <stephan@tmt.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "tasks/tasks.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <stdio.h>
#include "snd/soundHandler.h"
#include "ui/uiHandler.h"
#include "util/util.h"
#include "file/file.h"

extern int DBuf_init (long (*)(u32*, long), long (*)(u32*, long), int);
extern int DBuf_rerror(), DBuf_werror();
extern void DBuf_close();

static long soundreader (u32*, long), filewriter (u32*, long);

static const struct File* f;
static long samp_fac, read;
static int timer, b, closed, error;

int Task_record_init (int contd)
{
  int r, c;

  f = File_init (Config_buffilename());

  if (!contd) {
  	File_write_new (f);
  	read = 0;
  }
  else
  	File_append (f);

  error = 0;
  if (!File_ok (f)) {
  	error = 1;
  	UIHandler_msg (strcatalloc (_("Create failed: "), File_name (f)));
  	sleep (1);
  	return -1;
  }

  UIHandler_time (Time_i2clock (0));
  UIHandler_size (0, File_size (f));
  Format_init (f, contd);

  r = Config_iget ("SOUND_SPEED");
  b = Config_iget ("SOUND_BITS");
  c = Config_iget ("SOUND_CHANNELS");
  timer = Config_iget ("TIMER");
  
  Format_set_params (r, b, c, contd);
  
  SoundHandler_init (&r, &b, &c, 0);
  samp_fac = c*(b/8)*r; /* TODO: make this to what's really played */
  UIHandler_rbc (r, b, c);
  if (!UIHandler_is_commandline())
  	UIHandler_msg (_("Recording..."));

  /* two new threads are started and the remaining takes over */
  /* error checking and keeping the clock in sync             */
  if (DBuf_init (soundreader, filewriter, samp_fac) < 0)
  	Exception_fatal ("Init of buffer failed.  Enough memory?");
  	
  closed = 0;
  return 0;
}

void Task_record_finish (int contd)
{
  if (closed || error == 2) return;
  
  DBuf_close (1);
  if (contd) 
  	UIHandler_msg (_("Recording halted."));
  else
  	UIHandler_msg ("");
  SoundHandler_close();
  File_set_size (f, read*(b/8));
  Format_close();
  File_close (f);
  closed = 1;
}

/* this is called once in a while by the UI to sync the display and
   do error checking */
int Task_record_timeout()
{
  long seconds;
  static long oldseconds;

  if (error) return 0;  
  if (DBuf_rerror()) {
  	error = 1;
  	UIHandler_msg (_("Sound read error."));
  }
  if (DBuf_werror()) {
  	error = 1;
  	UIHandler_msg (_("File write error."));
  	Task_write();
  }
  
  seconds = (read * (b/8)) / samp_fac;
  if (seconds != oldseconds) {
  	UIHandler_time (Time_i2clock (seconds));
  	UIHandler_size (read*(b/8), 0);
  	oldseconds = seconds;
  }
  
  if (timer > 0 && seconds >= timer)
  	return 0;

  return !error;
}

/*------------------------------------------------------------------*/

static long soundreader (u32* p, long len)
{
  len = SoundHandler_rec_chunk (p, len);
  if (len <= 0) return -1;
  read += len;
  return len;
}

static long filewriter (u32* p, long len)
{
  static int count;
  Format_write_chunk (p, len);
  if ((++count % 5)== 0) sync();
  /* TODO: error checking */
  return 0;
}


